use std::net::IpAddr;

use tokio::signal;
// use tracing_subscriber::util::SubscriberInitExt;

use axum_with_seaorm::{APPLICATION_CONTEXT, init_context, router};
use axum_with_seaorm::config::app::AppConfig;

#[tokio::main]
async fn main() {
    // 初始化
    init_context().await;

    let cfg = APPLICATION_CONTEXT.get::<AppConfig>();
    let ip_addr = cfg.server.host.parse::<IpAddr>().unwrap();

    let app = router::init();
    let listener = tokio::net::TcpListener::bind((ip_addr, cfg.server.port)).await.unwrap();

    tracing::info!("Server runs on: {}", listener.local_addr().unwrap());

    axum::serve(listener, app).with_graceful_shutdown(shutdown_signal()).await.unwrap()
}


async fn shutdown_signal() {
    let ctrl_c = async {
        signal::ctrl_c()
            .await
            .expect("failed to install Ctrl+C handler");
    };

    #[cfg(unix)]
        let terminate = async {
        signal::unix::signal(signal::unix::SignalKind::terminate())
            .expect("failed to install signal handler")
            .recv()
            .await;
    };
    #[cfg(not(unix))]
        let terminate = std::future::pending::<()>();
    tokio::select! {
        _ = ctrl_c => {},
        _ = terminate => {},
    }
    tracing::info!("signal received, starting graceful shutdown");
}